@@ -3,7 +3,7 @@
 # =============================================================================
 #
 #    Remuco - A remote control system for media players.
-#    Copyright (C) 2006-2009 by the Remuco team, see AUTHORS.
+#    Copyright (C) 2006-2010 by the Remuco team, see AUTHORS.
 #
 #    This file is part of Remuco.
 #
@@ -72,22 +72,22 @@ Logging:
 #==============================================================================
 
 from remuco.adapter import PlayerAdapter, ItemAction, ListAction, ListReply
-from remuco.mpris import MPRISAdapter
 from remuco.config import Config
 from remuco.defs import *
 from remuco.manager import Manager
+from remuco.mpris import MPRISAdapter
 
 #==============================================================================
 # exports
 #==============================================================================
 
-__all__ = ("PlayerAdapter", "MPRISAdapter", "ListReply",
+__all__ = ["PlayerAdapter", "ListReply", "MPRISAdapter",
            "ItemAction", "ListAction", "Manager", "Config",
            
            "INFO_ALBUM", "INFO_ARTIST", "INFO_GENRE", "INFO_LENGTH",
            "INFO_RATING", "INFO_TAGS", "INFO_TITLE", "INFO_YEAR",
            
            "PLAYBACK_PAUSE", "PLAYBACK_PLAY", "PLAYBACK_STOP",
-           )
+           ]
 
 __version__ = REMUCO_VERSION
